# Generated by roxygen2: do not edit by hand

export(opts_read_geojson)
export(opts_read_json)
export(opts_write_geojson)
export(opts_write_json)
export(read_geojson_file)
export(read_geojson_str)
export(read_json_conn)
export(read_json_file)
export(read_json_raw)
export(read_json_str)
export(read_ndjson_file)
export(read_ndjson_str)
export(validate_json_file)
export(validate_json_str)
export(write_geojson_file)
export(write_geojson_str)
export(write_json_file)
export(write_json_str)
export(write_ndjson_file)
export(write_ndjson_str)
export(yyjson_read_flag)
export(yyjson_version)
export(yyjson_write_flag)
useDynLib(yyjsonr, .registration=TRUE)
